﻿using System.Collections.Generic;
using System.Threading.Tasks;
using GxPress.Auth;
using GxPress.Repository.Interface.Media;
using GxPress.Result.Media;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;

// For more information on enabling Web API for empty projects, visit https://go.microsoft.com/fwlink/?LinkID=397860

namespace GxPress.Api.AdminControllers
{
    [Route("api/admin/media")]
    [ApiController]
    [Authorize]
    public class AdminMediaController : Controller
    {
        private readonly IMediaRepository mediaRepository;
        private readonly ILoginContext _loginContext;
        public AdminMediaController(IMediaRepository mediaRepository, ILoginContext _loginContext)
        {
            this.mediaRepository = mediaRepository;
            this._loginContext = _loginContext;
        }
        /// <summary>
        /// 详情
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        [HttpGet("{id}")]
        public async Task<MediaResult> GetAsync(int id)
        {
            return await mediaRepository.GetAsync(id);
        }
        /// <summary>
        /// 删除
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        [HttpDelete("{id}")]
        public async Task<bool> DeleteAsync(int id)
        {
            return await mediaRepository.DeleteAsync(id);
        }
        /// <summary>
        /// 添加媒体
        /// </summary>
        /// <param name="result"></param>
        /// <returns></returns>
        [HttpPost]
        public async Task<bool> InsertAsync(MediaResult result)
        {
            result.AdminId = _loginContext.AccountId;
            result.Creator = _loginContext.Name;
            return await mediaRepository.InsertAsync(result);
        }
        /// <summary>
        /// 修改
        /// </summary>
        /// <param name="result"></param>
        /// <returns></returns>
        [HttpPut]
        public async Task<bool> UpdateAsync(MediaResult result)
        {
            return await mediaRepository.UpdateAsync(result);
        }
        /// <summary>
        /// 查询
        /// </summary>
        /// <param name="mediaType"></param>
        /// <returns></returns>
        [HttpPost("list/{mediaType}")]
        public async Task<IEnumerable<Entity.tede2.Media.Media>> GetAllAsync(int mediaType)
        {
            return await mediaRepository.GetAllAsync(mediaType);
        }
    }
}
