﻿using System.Linq;
using System.Collections.Generic;
using System.Threading.Tasks;
using AutoMapper;
using Datory;
using GxPress.Common.AppOptions;
using GxPress.Common.Tools;
using GxPress.Repository.Interface.Media;
using GxPress.Result.Media;
using Microsoft.Extensions.Options;
using System.Transactions;

namespace GxPress.Repository.Implement.Media
{
    public class MediaRepository : IMediaRepository
    {
        private readonly Repository<Entity.tede2.Media.Media> _repository;
        private readonly Repository<Entity.tede2.Media.MediaLibrary> mediaLibraryRepository;
        private readonly IMapper _mapper;
        private readonly string _connectionString;
        private readonly string _databaseTypestr;
        public MediaRepository(IOptionsMonitor<DatabaseOptions> dbOptionsAccessor, IMapper mapper)
        {
            _databaseTypestr = dbOptionsAccessor.CurrentValue.DatabaseType;
            _connectionString = dbOptionsAccessor.CurrentValue.ConnectionString;
            var databaseType =
                StringUtils.ToEnum<DatabaseType>(dbOptionsAccessor.CurrentValue.DatabaseType, DatabaseType.MySql);
            var database = new Database(databaseType, dbOptionsAccessor.CurrentValue.ConnectionString);
            _repository = new Repository<Entity.tede2.Media.Media>(database);
            mediaLibraryRepository = new Repository<Entity.tede2.Media.MediaLibrary>(database);
            _mapper = mapper;
        }

        public IDatabase Database => _repository.Database;
        public string TableName => _repository.TableName;
        public List<TableColumn> TableColumns => _repository.TableColumns;

        public async Task<MediaResult> GetAsync(int id)
        {
            var result = new MediaResult();
            //获取媒体
            var media = await _repository.GetAsync(id);
            result = _mapper.Map<MediaResult>(media);
            //获取媒体库资源
            var mediaLibrarys = await mediaLibraryRepository.GetAllAsync(Q.Where(nameof(Entity.tede2.Media.MediaLibrary.MediaId), id));
            result.MediaLibraryRequest = mediaLibrarys.ToList().Select(n => _mapper.Map<MediaLibraryResult>(n)).ToList();
            return result;
        }

        public async Task<bool> DeleteAsync(int id)
        {
            return await _repository.DeleteAsync(id);
        }
        /// <summary>
        /// 添加媒体
        /// </summary>
        /// <param name="result"></param>
        /// <returns></returns>
        public async Task<bool> InsertAsync(MediaResult result)
        {
            var model = _mapper.Map<Entity.tede2.Media.Media>(result);
            try
            {
                using (var transactionScope = new TransactionScope())
                {
                    var id = await _repository.InsertAsync(model);
                    if (id > 0)
                    {
                        foreach (var item in result.MediaLibraryRequest)
                        {
                            var mediaLibrary = _mapper.Map<Entity.tede2.Media.MediaLibrary>(item);
                            await mediaLibraryRepository.InsertAsync(mediaLibrary);
                        }
                    }
                    if(result.Id==0&&result.TeacherId==0&&!string.IsNullOrEmpty(result.Author)){
                        //新增名师
                        
                    }
                    //提交事务
                    transactionScope.Complete();
                }
            }
            catch (System.Exception ex)
            {
                throw new Common.Exceptions.BusinessException(ex.Message);
            }

            return true;
        }

        public async Task<bool> UpdateAsync(Entity.tede2.Media.Media note)
        {
            return await _repository.UpdateAsync(note);
        }
        public async Task<bool> UpdateAsync(SqlKata.Query query)
        {
            return await _repository.UpdateAsync(query) > 0;
        }

        public async Task<IEnumerable<Entity.tede2.Media.Media>> GetAllAsync(int mediaType)
        {
            return await _repository.GetAllAsync(Q.Where(nameof(Entity.tede2.Media.Media.MediaType), mediaType).OrderByDesc(nameof(Entity.tede2.Media.Media.Sort)));
        }

        public async Task<bool> UpdateAsync(MediaResult result)
        {
            var model = _mapper.Map<Entity.tede2.Media.Media>(result);
            try
            {
                using (var transactionScope = new TransactionScope())
                {
                    await _repository.UpdateAsync(model);
                    //删除媒体库
                    await mediaLibraryRepository.DeleteAsync(Q.Where(nameof(Entity.tede2.Media.MediaLibrary.MediaId), result.Id));
                    foreach (var item in result.MediaLibraryRequest)
                    {
                        var mediaLibrary = _mapper.Map<Entity.tede2.Media.MediaLibrary>(item);
                        await mediaLibraryRepository.InsertAsync(mediaLibrary);
                    }
                    //提交事务
                    transactionScope.Complete();
                }
            }
            catch (System.Exception ex)
            {
                throw new Common.Exceptions.BusinessException(ex.Message);
            }

            return true;
        }
    }
}
